/***************************************************************************
 *                                                                         *
 *   KCPULoad and KNetLoad are copyright (c) 1999-2000, Markus Gustavsson  *
 *                                       (c) 2002, Ben Burton              *
 *                                       (c) 2004, Diego Petten           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "statdock.h"
#include "statpopup.h"
#include <qpainter.h>

const QColor StatDock::colorGrid(120, 120, 120);
const QColor StatDock::colorGridInactive(60, 60, 60);
const QColor StatDock::colorLabel(255, 255, 255);
const QColor StatDock::colorLabelInactive(125, 125, 125);
const QColor StatDock::colorLower(255, 255, 255);
const QColor StatDock::colorLowerInactive(125, 125, 125);

#define DOCK_SIZE 24
#define DOCK_SCALE 4.34783 // Approx. 100/23.
#define SOFT_STEP 3

StatDock::StatDock(int whichDock, const char* useLabel,
        StatPopup *parent, const char *name) :
        KSystemTray(parent,name),
        label(QString::fromUtf8(useLabel)),
        numDock(whichDock),
        bufUpper(new int[DOCK_SIZE]),
        bufLower(new int[DOCK_SIZE]),
        pos(0) {
    // Initialise the stored readings.
    memset(bufUpper, 0, DOCK_SIZE*sizeof(int));
    memset(bufLower, 0, DOCK_SIZE*sizeof(int));
    
    i = DOCK_SIZE-1;

    // Initialise the display.
    parent->initDock(this, contextMenu());
    if ( ! Settings::transparentBackground() )
        setBackgroundColor(Settings::colorBackground());
    resize(24, 24);
    show();
}

StatDock::~StatDock() {
    delete[] bufUpper;
    delete[] bufLower;
}

void StatDock::setLabel(const char* set) {
    label = set;
    repaint();
}

void StatDock::clearHistory(void) {
    for (i = 0; i < DOCK_SIZE; i++)
        bufUpper[i] = bufLower[i] = 0;
    repaint();
}

QColor StatDock::getColor()
{
	QColor base;
	if ( ! numDock )
		base = Settings::color1();
	else
		base = Settings::color2();

	return Settings::enabled() ? base : base.dark();	
}

void StatDock::addPercentReading(int upper, int lower) {
    // Rescale the readings to a measure in pixels.
    upper = (int) (((float) upper) / DOCK_SCALE);
#if STATPOPUP_SPLITTABLE == 1
    lower = (Settings::split() ? (int) (((float) lower) / DOCK_SCALE) : 0);
#else
    lower = 0;
#endif

    if (Settings::soft()) {
        // Do a bit of juggling to obtain a soft reading.
        oldUpper = bufUpper[pos];
        oldLower = bufLower[pos];
        if(++pos >= DOCK_SIZE)
            pos = 0;

        // Modify the upper reading.
        if (upper > oldUpper + SOFT_STEP)
            upper = oldUpper + SOFT_STEP;
        else if (upper < oldUpper - SOFT_STEP)
            upper = oldUpper - SOFT_STEP;
        bufUpper[pos] = upper;

#if STATPOPUP_SPLITTABLE == 1
        // Modify the lower reading.
        if (Settings::split()) {
            if (lower > oldLower + SOFT_STEP)
                lower = oldLower + SOFT_STEP;
            else if (lower < oldLower - SOFT_STEP)
                lower = oldLower - SOFT_STEP;
        }
#endif
        bufLower[pos] = lower;
    } else {
        // Just a straight update.
        if(++pos >= DOCK_SIZE)
            pos = 0;
        bufUpper[pos] = upper;
        bufLower[pos] = lower;
    }

    // Refresh the diagram.
    repaint();
}

void StatDock::paintEvent(QPaintEvent*) {
    QPainter p(this);

    // Start by drawing the grid.
    if(Settings::grid()) {
        p.setPen(Settings::enabled() ? colorGrid : colorGridInactive);
        p.drawLine(0, 4, 23, 4);
        p.drawLine(0, 9, 23, 9);
        p.drawLine(0, 14, 23, 14);
        p.drawLine(0, 19, 23, 19);
    }
    
    // Avoid call of the function in a loop
    QColor bgColor = Settings::colorBackground();
    QColor color = getColor();

    if(Settings::style() == Settings::EnumStyle::Shades) {
        // Shaded
        for(i = 0; i < 24; i++) {
            tmpPos = (pos + i + 1) % DOCK_SIZE;
            for(j = 0; j <= bufUpper[tmpPos]; j++) {
                if (bufUpper[tmpPos] == 0 || j == 0)
                    p.setPen(bgColor);
                else if (Settings::enabled())
                    p.setPen(color.dark((100 * bufUpper[tmpPos]) / j));
                else
                    p.setPen(color.dark((200 * bufUpper[tmpPos]) / j));

#if STATPOPUP_SPLITTABLE == 1
                if(Settings::split())
                    p.drawPoint(i, DOCK_SIZE - 1 - j - bufLower[tmpPos]);
                else
#endif
                    p.drawPoint(i, DOCK_SIZE - j);
            }

#if STATPOPUP_SPLITTABLE == 1
            if(Settings::split()) {
                for(j = 0; j <= bufLower[tmpPos]; j++) {
                    if (bufLower[tmpPos] == 0 || j == 0)
                        p.setPen(bgColor);
                    else if (Settings::enabled())
                        p.setPen(colorLower.dark((100 * bufLower[tmpPos]) / j));
                    else
                        p.setPen(colorLower.dark((200 * bufLower[tmpPos]) / j));

                    p.drawPoint(i, 23 - j);
                }
            }
#endif
        }
    } else if (Settings::style() == Settings::EnumStyle::Bars) {
        // Bars
#if STATPOPUP_SPLITTABLE == 1
        if(Settings::split()) {
            // Draw the upper bars, then the lower to save on pen
            // adjustments.
            p.setPen(color);
            for(i = 0; i < 24; i++) {
                tmpPos = (pos + i + 1) % DOCK_SIZE;
                p.drawLine(i, DOCK_SIZE - 1 - bufUpper[tmpPos] -
                    bufLower[tmpPos], i, DOCK_SIZE - 1 - bufLower[tmpPos]);
            }
            p.setPen(Settings::enabled() ? colorLower : colorLowerInactive);
            for(i = 0; i < 24; i++) {
                tmpPos = (pos + i + 1) % DOCK_SIZE;
                p.drawLine(i, DOCK_SIZE - 1 - bufLower[tmpPos],
                    i, DOCK_SIZE - 1);
            }
        } else
#endif
        {
            p.setPen(color);
            for(i = 0; i < 24; i++) {
                tmpPos = (pos + i + 1) % DOCK_SIZE;
                p.drawLine(i, DOCK_SIZE - 1 - bufUpper[tmpPos],
                    i, DOCK_SIZE - 1);
            }
        }
    } else {
        // Lines
#if STATPOPUP_SPLITTABLE == 1
        if(Settings::split()) {
            // Draw the upper line, then the lower to save on pen
            // adjustments.
            p.setPen(getColor());
            for(i = 0; i < 24; i++) {
                tmpPos = (pos + i + 1) % DOCK_SIZE;
                p.drawPoint(i, DOCK_SIZE - 1 -
                    bufUpper[tmpPos] - bufLower[tmpPos]);
            }
            p.setPen(Settings::enabled() ? colorLower : colorLowerInactive);
            for(i = 0; i < 24; i++) {
                tmpPos = (pos + i + 1) % DOCK_SIZE;
                p.drawPoint(i, DOCK_SIZE - 1 - bufLower[tmpPos]);
            }
        } else
#endif
        {
            p.setPen(getColor());
            for(i = 0; i < 24; i++) {
                tmpPos = (pos + i + 1) % DOCK_SIZE;
                p.drawPoint(i, DOCK_SIZE - 1 - bufUpper[tmpPos]);
            }
        }
    }

    // Finally label the diagrams.
    if(Settings::labelled()) {
        p.setFont(QFont("Helvetica", 8));
        p.setPen((Settings::enabled()) ? colorLabel : colorLabelInactive);
        p.drawText(rect(), AlignLeft | AlignTop, label);
    }
}

#include "statdock.moc"
